import 'package:flutter/widgets.dart';
import 'package:flutter/material.dart';
// 3rd package
import 'package:connectivity/connectivity.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';

class LoableList extends StatefulWidget {
  final String search;
  final Widget empty;
  final Widget loading;
  final Function child;
  final Function onFetch;

  LoableList({
    Key key,
    this.child,
    this.empty,
    this.search,
    this.loading,
    this.onFetch,
  }) : super(key: key);

  @override
  _DataListState createState() => _DataListState();
}

class _DataListState extends State<LoableList> {
  final snackBar = SnackBar(
    duration: Duration(days: 1),
    backgroundColor: Colors.red,
    content: Text(
      'No internet Connection',
      style: TextStyle(color: Colors.white),
    ),
  );

  bool canLoad = true;
  int defaultLimit = 30, limit = 30;
  Future<List<dynamic>> _futureData;

  RefreshController _refreshController = RefreshController(
    initialRefresh: false,
  );

  @override
  void initState() {
    super.initState();
    _checkConnection();
    _futureData = widget.onFetch(limit, widget.search);
  }

  void _checkConnection() async {
    var connectivityResult = await (Connectivity().checkConnectivity());
    if (connectivityResult == ConnectivityResult.none) {
      Scaffold.of(context).showSnackBar(snackBar);
    }
  }

  void _onRefresh() async {
    setState(() {
      limit = defaultLimit;
    });
    await Future.delayed(Duration(milliseconds: 1000));
    _futureData = widget.onFetch(defaultLimit, widget.search);
    _refreshController.refreshCompleted();
  }

  void _onLoading() async {
    if (!canLoad) return _refreshController.loadNoData();
    await Future.delayed(Duration(milliseconds: 1000));

    _futureData = widget.onFetch(limit + defaultLimit, widget.search);
    if (mounted)
      setState(() {
        limit = limit + defaultLimit;
      });
    _refreshController.loadComplete();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<List<dynamic>>(
      future: _futureData,
      builder: (context, snapshot) {
        if (snapshot.hasData) {
          List<dynamic> data = snapshot.data;
          return _listView(context, data);
        } else if (snapshot.hasError) {
          return result(true);
        }
        return result(false);
      },
    );
  }

  Widget _listView(BuildContext context, dynamic data) {
    return SmartRefresher(
      enablePullUp: true,
      enablePullDown: true,
      onRefresh: _onRefresh,
      onLoading: _onLoading,
      child: widget.child(context, data),
      controller: _refreshController,
      header: MaterialClassicHeader(),
    );
  }

  Widget result(empty) {
    return Container(
        child: Center(
      child: empty ? Text('No result found!') : CircularProgressIndicator(),
    ));
  }
}
