import 'package:flutter/material.dart';

class PopupMenu extends StatefulWidget {
  final String value;
  final Widget child;
  final bool showCheck;
  final Function onChange;
  final List<MenuOption> options;

  PopupMenu({
    Key key,
    this.value,
    this.options,
    this.onChange,
    this.showCheck = false,
    this.child = const Icon(Icons.more_vert),
  }) : super(key: key);

  @override
  _IPopupMenuState createState() => _IPopupMenuState();
}

class _IPopupMenuState extends State<PopupMenu> {
  String value = '';

  @override
  void initState() {
    loadInitValue();
    super.initState();
  }

  void loadInitValue() {
    if (widget.value != null) {
      setState(() {
        value = widget.value;
      });
    }
  }

  void _select(MenuOption item) {
    setState(() {
      value = item.value;
    });
    if (item.onTap != null) return item.onTap();
    if (item.option == null) {
      widget.onChange(item.value);
    } else {
      widget.onChange(item.value, item.option);
    }
  }

  @override
  Widget build(BuildContext context) {
    return PopupMenuButton<MenuOption>(
      icon: widget.child,
      onSelected: _select,
      itemBuilder: (BuildContext context) {
        return widget.options.map((MenuOption item) {
          return PopupMenuItem<MenuOption>(
            value: item,
            child: menuItem(item),
          );
        }).toList();
      },
    );
  }

  Widget menuItem(dynamic item) {
    bool dart = Theme.of(context).brightness == Brightness.dark;
    if (value != item.value) return textIcon(item, dart);

    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: <Widget>[
        textIcon(item, dart),
        Icon(Icons.check, color: dart ? Colors.white : Colors.black)
      ],
    );
  }

  Widget textIcon(dynamic item, bool dart) {
    if (item.icon == null) return Text(item.label);

    return Row(
      children: <Widget>[
        Icon(item.icon, color: dart ? Colors.white : Colors.black),
        SizedBox(width: 20),
        Text(item.value),
      ],
    );
  }
}

class MenuOption {
  final String value;
  final String label;
  final IconData icon;
  final Function onTap;
  final dynamic option;

  MenuOption({
    this.icon,
    this.onTap,
    this.value,
    this.label,
    this.option,
  });
}
